@@ -118,7 +118,7 @@ static __inline__ CARD32 F_TO_DW(float val)
 /* Assumes that depth 15 and 16 can be used as depth 16, which is okay since we
  * require src and dest datatypes to be equal.
  */
-static Bool RADEONGetDatatypeBpp(int bpp, CARD32 *type)
+Bool RADEONGetDatatypeBpp(int bpp, CARD32 *type)
 {
 	switch (bpp) {
 	case 8:
@@ -157,7 +157,7 @@ static Bool RADEONGetOffsetPitch(PixmapPtr pPix, int bpp, CARD32 *pitch_offset,
 {
 	RINFO_FROM_SCREEN(pPix->drawable.pScreen);
 
-	if (pitch % info->exa->pixmapPitchAlign != 0)
+	if (pitch > 16320 || pitch % info->exa->pixmapPitchAlign != 0)
 		RADEON_FALLBACK(("Bad pitch 0x%08x\n", pitch));
 
 	if (offset % info->exa->pixmapOffsetAlign != 0)
@@ -172,7 +172,7 @@ static Bool RADEONGetOffsetPitch(PixmapPtr pPix, int bpp, CARD32 *pitch_offset,
 	return TRUE;
 }
 
-static Bool RADEONGetPixmapOffsetPitch(PixmapPtr pPix, CARD32 *pitch_offset)
+Bool RADEONGetPixmapOffsetPitch(PixmapPtr pPix, CARD32 *pitch_offset)
 {
 	RINFO_FROM_SCREEN(pPix->drawable.pScreen);
 	CARD32 pitch, offset;
@@ -197,7 +197,7 @@ static Bool RADEONPrepareAccess(PixmapPtr pPix, int index)
     RINFO_FROM_SCREEN(pPix->drawable.pScreen);
     unsigned char *RADEONMMIO = info->MMIO;
     CARD32 offset = exaGetPixmapOffset(pPix);
-    int bpp, rc, soff;
+    int bpp, soff;
     CARD32 size, flags;
 
     /* Front buffer is always set with proper swappers */
@@ -231,6 +231,7 @@ static Bool RADEONPrepareAccess(PixmapPtr pPix, int index)
 #if defined(XF86DRI)
     if (info->directRenderingEnabled && info->allowColorTiling) {
 	drmRadeonSurfaceAlloc drmsurfalloc;
+	int rc;
 
         drmsurfalloc.address = offset;
         drmsurfalloc.size = size;
@@ -292,27 +293,34 @@ static void RADEONFinishAccess(PixmapPtr pPix, int index)
 
 #define RADEON_SWITCH_TO_2D()						\
 do {									\
-        /*if (info->engineMode == EXA_ENGINEMODE_2D)*/                      \
-            /*break;*/                                                      \
-	BEGIN_ACCEL(2);							\
-	OUT_ACCEL_REG(RADEON_RB2D_DSTCACHE_CTLSTAT,  RADEON_RB2D_DC_FLUSH); \
-	OUT_ACCEL_REG(RADEON_WAIT_UNTIL,				\
-		RADEON_WAIT_HOST_IDLECLEAN |				\
-		RADEON_WAIT_3D_IDLECLEAN);				\
+	CARD32 wait_until = 0;			\
+	BEGIN_ACCEL(1);							\
+	switch (info->engineMode) {					\
+	case EXA_ENGINEMODE_UNKNOWN:					\
+	    wait_until |= RADEON_WAIT_HOST_IDLECLEAN | RADEON_WAIT_2D_IDLECLEAN;	\
+	case EXA_ENGINEMODE_3D:						\
+	    wait_until |= RADEON_WAIT_3D_IDLECLEAN;			\
+	case EXA_ENGINEMODE_2D:						\
+	    break;							\
+	}								\
+	OUT_ACCEL_REG(RADEON_WAIT_UNTIL, wait_until);			\
 	FINISH_ACCEL();							\
         info->engineMode = EXA_ENGINEMODE_2D;                           \
 } while (0);
 
 #define RADEON_SWITCH_TO_3D()						\
 do {									\
-        /*if (info->engineMode == EXA_ENGINEMODE_3D)*/                      \
-            /*break;*/                                                      \
-	BEGIN_ACCEL(2);							\
-	OUT_ACCEL_REG(RADEON_RB2D_DSTCACHE_CTLSTAT,  RADEON_RB2D_DC_FLUSH); \
-	OUT_ACCEL_REG(RADEON_WAIT_UNTIL,				\
-		RADEON_WAIT_HOST_IDLECLEAN |				\
-		RADEON_WAIT_2D_IDLECLEAN |				\
-		RADEON_WAIT_3D_IDLECLEAN);				\
+	CARD32 wait_until = 0;			\
+	BEGIN_ACCEL(1);							\
+	switch (info->engineMode) {					\
+	case EXA_ENGINEMODE_UNKNOWN:					\
+	    wait_until |= RADEON_WAIT_HOST_IDLECLEAN | RADEON_WAIT_3D_IDLECLEAN;	\
+	case EXA_ENGINEMODE_2D:						\
+	    wait_until |= RADEON_WAIT_2D_IDLECLEAN;			\
+	case EXA_ENGINEMODE_3D:						\
+	    break;							\
+	}								\
+	OUT_ACCEL_REG(RADEON_WAIT_UNTIL, wait_until);			\
 	FINISH_ACCEL();							\
         info->engineMode = EXA_ENGINEMODE_3D;                           \
 } while (0);
@@ -366,9 +374,9 @@ Bool RADEONSetupMemEXA (ScreenPtr pScreen)
 {
     ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
     RADEONInfoPtr info = RADEONPTR(pScrn);
+    xf86CrtcConfigPtr   xf86_config = XF86_CRTC_CONFIG_PTR(pScrn);
     int cpp = info->CurrentLayout.pixel_bytes;
-    int l;
-    int next, screen_size;
+    int screen_size;
     int byteStride = pScrn->displayWidth * cpp;
 
     if (info->exa != NULL) {
@@ -394,23 +402,45 @@ Bool RADEONSetupMemEXA (ScreenPtr pScreen)
     xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Allocating from a screen of %ld kb\n",
 	       info->exa->memorySize / 1024);
 
-    xf86DrvMsg(pScrn->scrnIndex, X_INFO,
-	       "Will use %d kb for front buffer at offset 0x%08x\n",
-	       screen_size / 1024, 0);
+
+    /* Reserve static area for hardware cursor */
+    if (!xf86ReturnOptValBool(info->Options, OPTION_SW_CURSOR, FALSE)) {
+	int cursor_size = 64 * 4 * 64;
+	int c;
+
+	for (c = 0; c < xf86_config->num_crtc; c++) {
+	    xf86CrtcPtr crtc = xf86_config->crtc[c];
+	    RADEONCrtcPrivatePtr radeon_crtc = crtc->driver_private;
+
+	    radeon_crtc->cursor_offset = info->exa->offScreenBase;
+	    info->exa->offScreenBase += cursor_size;
+
+	    xf86DrvMsg(pScrn->scrnIndex, X_INFO,
+		       "Will use %d kb for hardware cursor %d at offset 0x%08x\n",
+		       (cursor_size * xf86_config->num_crtc) / 1024,
+		       c,
+		       (unsigned int)radeon_crtc->cursor_offset);
+	}
+
+
+    }
 
 #if defined(XF86DRI)
     if (info->directRenderingEnabled) {
-	int depth_size;
+	int depthCpp = (info->depthBits - 8) / 4, l, next, depth_size;
 
 	info->frontOffset = 0;
 	info->frontPitch = pScrn->displayWidth;
 
+        xf86DrvMsg(pScrn->scrnIndex, X_INFO,
+	       "Will use %d kb for front buffer at offset 0x%08x\n",
+	       screen_size / 1024, info->frontOffset);
 	RADEONDRIAllocatePCIGARTTable(pScreen);
 	
 	if (info->cardType==CARD_PCIE)
 	  xf86DrvMsg(pScrn->scrnIndex, X_INFO,
 		     "Will use %d kb for PCI GART at offset 0x%08x\n",
-		     RADEON_PCIGART_TABLE_SIZE / 1024,
+		     info->pciGartSize / 1024,
 		     (int)info->pciGartOffset);
 
 	/* Reserve a static area for the back buffer the same size as the
@@ -435,7 +465,7 @@ Bool RADEONSetupMemEXA (ScreenPtr pScreen)
 	 * handle tiling.
 	 */
 	info->depthPitch = RADEON_ALIGN(pScrn->displayWidth, 32);
-	depth_size = RADEON_ALIGN(pScrn->virtualY, 16) * info->depthPitch * cpp;
+	depth_size = RADEON_ALIGN(pScrn->virtualY, 16) * info->depthPitch * depthCpp;
 	next = RADEON_ALIGN(info->exa->offScreenBase, RADEON_BUFFER_ALIGN);
 	if (next + depth_size <= info->exa->memorySize)
 	{
@@ -463,9 +493,12 @@ Bool RADEONSetupMemEXA (ScreenPtr pScreen)
 	    /* Minimum texture size is for 2 256x256x32bpp textures */
 	    info->textureSize = 0;
 	}
-    }
+    } else
 #endif /* XF86DRI */
-	
+    	xf86DrvMsg(pScrn->scrnIndex, X_INFO,
+		       "Will use %d kb for front buffer at offset 0x%08x\n",
+		       screen_size / 1024, 0);
+
     xf86DrvMsg(pScrn->scrnIndex, X_INFO,
 	       "Will use %ld kb for X Server offscreen at offset 0x%08lx\n",
 	       (info->exa->memorySize - info->exa->offScreenBase) /
@@ -473,3 +506,20 @@ Bool RADEONSetupMemEXA (ScreenPtr pScreen)
 
     return TRUE;
 }
+
+#ifdef XF86DRI
+
+#ifndef ExaOffscreenMarkUsed
+extern void ExaOffscreenMarkUsed(PixmapPtr);
+#endif
+
+unsigned long long
+RADEONTexOffsetStart(PixmapPtr pPix)
+{
+    exaMoveInPixmap(pPix);
+    ExaOffscreenMarkUsed(pPix);
+
+    return RADEONPTR(xf86Screens[pPix->drawable.pScreen->myNum])->fbLocation +
+	exaGetPixmapOffset(pPix);
+}
+#endif
